using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.DirectX;
using Microsoft.DirectX.Direct3D;

namespace Driver
{
    class Car
    {
        public const float Height = 2.5f;
        public const float Depth = 6.0f;
        public const float SpeedIncrement = 0.1f;
        public const float Scale = 0.05f;

        private float carLocation = Driver.RoadLocationLeft;
        private float carDiameter;
        private float carSpeed = 20.0f;

        private bool movingLeft = false;
        private bool movingRight = false;

        private bool Accelerate = false;
        private bool Break = false;

        private Mesh carMesh = null;
        private Material[] carMaterials = null;
        private Texture[] carTextures = null;

        public Car(Device device) 
        {
            carMesh = Driver.LoadMesh(device, "car.x", ref carMaterials, ref carTextures);

            VertexBuffer vb = carMesh.VertexBuffer;

            try
            {
                GraphicsStream stm = vb.Lock(0, 0, LockFlags.None);
                Vector3 center;

                float radius = Geometry.ComputeBoundingSphere(stm, carMesh.NumberVertices, carMesh.VertexFormat, out center);
                carDiameter = (radius * 2) * Scale;

            }
            finally
            {
                vb.Unlock();
                vb.Dispose();
            }
        }

        public void Draw(Device device)
        {
            device.Transform.World = Matrix.Scaling(Scale, Scale, Scale) * Matrix.Translation(carLocation, Height, Depth);

            for (int i = 0; i < carMaterials.Length; i++)
            {
                device.Material = carMaterials[i];
                device.SetTexture(0, carTextures[i]);
                carMesh.DrawSubset(i);
            }
        }

        public float Location
        {
            get { return carLocation; }
            set { carLocation = value; }
        }
        public float Diameter
        {
            get { return carDiameter; }
        }
        public float Speed
        {
            get { return carSpeed; }
            set { carSpeed = value; }
        }
        public bool IsMovingLeft
        {
            get { return movingLeft; }
            set { movingLeft = value; }
        }
        public bool IsMovingRight
        {
            get { return movingRight; }
            set { movingRight = value; }
        }

        public void Update(float elapsedTime)
        {
            if (movingLeft)
            {
                carLocation += carSpeed * elapsedTime;

                if (carLocation >= Driver.RoadLocationLeft)
                {
                    movingLeft = false;
                    carLocation = Driver.RoadLocationLeft;
                }
            }

            if (movingRight)
            {
                carLocation -= carSpeed * elapsedTime;

                if (carLocation <= Driver.RoadLocationRight)
                {
                    movingRight = false;
                    carLocation = Driver.RoadLocationRight;
                }
            }
        
        }

    }
}
